#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include <sys/param.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "addr_range.h"

static unsigned long first_addr(addr_range *ar);
static unsigned long last_addr(addr_range *ar);  

/*------------------------------------------------------------------*/
static int overlap(addr_range *ar1, addr_range *ar2) {

    if((ntohl(ar1->r.first) < ntohl(ar2->r.first) && ntohl(ar1->r.last) < ntohl(ar2->r.first)) ||
       (ntohl(ar1->r.last) > ntohl(ar2->r.last) && ntohl(ar1->r.first) > ntohl(ar2->r.last))) {
        return 0;  /* does not overlap */
    }
    return 1; /* overlap */
  
}
/*------------------------------------------------------------------*/
static addr_range_raw raw(addr_range *ar) {
    return ar->r;
}
/*------------------------------------------------------------------*/
static unsigned int end_time_func(addr_range *ar) {
    return ntohl(ar->r.end_time);
}
/*------------------------------------------------------------------*/

static unsigned long first_addr(addr_range *ar){

    return ar->r.first;
}

/*------------------------------------------------------------------*/

static unsigned long last_addr(addr_range *ar){

    return ar->r.last;
}

/*------------------------------------------------------------------*/
static void adjust(addr_range *ar, int skew) {
    ar->r.end_time = htonl(ntohl(ar->r.end_time) - skew);
}
/*------------------------------------------------------------------*/
static void delete(addr_range *ar) {
    free(ar);
}
/*------------------------------------------------------------------*/
static void debug_print(addr_range *ar) {
    struct in_addr first, last;
    time_t time;
    char buf[200];
    
    first.s_addr = ar->r.first;
    last.s_addr  = ar->r.last;
    time = (time_t)ntohl(ar->r.end_time);
  
    strcpy(buf, inet_ntoa(first));
    printf("From %s to %s, end time %s", buf , inet_ntoa(last),
           ctime(&time));
}
/*------------------------------------------------------------------*/
addr_range *addr_range_new(unsigned long first, unsigned long last,
			   unsigned int end_time) {
    addr_range *ar = malloc(sizeof(addr_range));
    ar->r.first = first;
    ar->r.last  = last;
    ar->r.end_time = htonl(end_time);

    ar->end_time = end_time_func;
    ar->raw     = raw;
    ar->overlap = overlap;

    ar->first_addr = first_addr;
    ar->last_addr = last_addr;

    ar->adjust = adjust;
  
    ar->delete = delete;
    ar->debug_print = debug_print;
    return ar;
}


/*------------------------------------------------------------------*/
addr_range *addr_range_new_from_raw(addr_range_raw *r) {
  
    addr_range *ar = malloc(sizeof(addr_range));
    ar->r.first = r->first;
    ar->r.last  = r->last;
    ar->r.end_time = r->end_time;

    ar->end_time = end_time_func;
    ar->raw     = raw;
    ar->overlap = overlap;
    ar->first_addr = first_addr;
    ar->last_addr = last_addr;

    ar->adjust = adjust;
  
    ar->delete = delete;
    ar->debug_print = debug_print;
    return ar;
}
