#ifndef __PLIST_H__
#define __PLIST_H__

/**
 * plist.h
 *
 * Implements a priority queue.
 *
 * Methods are
 *
 * int insert(plist *p, int priority, void *item);
 *
 *   Inserts an item item with priority priority into queue p.
 *
 * void *first(plist *p);
 *
 *   Returns the first item (with lowest priority) in queue p.
 *
 * int priority(plist *p);
 *
 *   Returns the priority of the frontmost item.
 *
 * int remove(plist *p);
 *
 *   Removes the frontmost item.
 *
 * void remove_element(plist *p, int (* matching_function)(void *element));
 *
 *   Removes all elements that match a given pattern. The function
 *   matching_function(), which is supplied by the user, is invoked
 *   on all elements and if the function returns != 0, the element
 *   is removed.
 *
 * int is_empty(plist *p);
 *
 *   Returns 1 if the queue is empty.
 *
 */
 

typedef struct plist_ plist;

struct plist_node {
  unsigned int priority;
  void *item;
  struct plist_node *next;
};


struct plist_ {

  struct plist_node *head;
  int len;

  /* these two variables are used for internal state,
     related to performance improvements */
  int last_pos;
  struct plist_node *last_node;
  
  int (* insert)(plist *p, unsigned int priority, void *item);
  void * (* first)(plist *p);
  unsigned int (* priority)(plist *p, int pos);
  int (* remove)(plist *p, int pos);

  void (* remove_element)(plist *p, int (* matching_function)(void *obj, void *element), void *obj);

  int (* length)(plist *p);
  int (* find)(plist *p, void *item);
  void *(* elem_at)(plist *p, int pos);  
  
  int (* is_empty)(plist *p);

  void (* delete)(plist *p);
};

plist *plist_new();

#endif
