/* This file implements a useful debugging routine, and it is not a necessary
 * part of the MAAS design.
 *
 * The debugging routines allows the user/programmer to turn on selected
 * debugging messages, controllable from the command line arguments passed to
 * MAAS (-d).
 *
 * The debugging flags are defined in debug.h.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */

#include <string.h>
#include "debug.h"

/* A string of enabled flags. Controls which DEBUG messages are printed. */
static char *enable_flags = NULL;

/**
 * Initialize so that only DEBUG messages with a flag in flag_list will be
 * printed. If the flag is "+", we enable all DEBUG messages.
 *
 * @param flag_list A string of characters for whose DEBUG messages are
 * to be enabled.
 */
void DEBUG_INIT(char *flag_list) {
    enable_flags = flag_list;
}

/**
 * Check if debug flag is enabled.
 *
 * @param flag The flag to check.
 * @return If flag not set FAILURE(-1) is returned, else SUCCESS(0) is returned.
 */
int DEBUG_ENABLED(char flag) {
    if (enable_flags != NULL) {
        if ((strchr(enable_flags, flag) != 0)
            || (strchr(enable_flags, '+') != 0)) {
            return SUCCESS;
        }
    }
    
    return FAILURE;
}

/**
 * Print debug message if flag is enabled.
 * Works as printf, but with an extra first argument.
 *
 * @param flag The flag that has to be set for this debug message to be printed.
 */
void DEBUG(char flag, char *format, ...) {
    if (DEBUG_ENABLED(flag) == SUCCESS) {
        va_list ap;
        va_start(ap, format);
        vfprintf(stdout, format, ap);
        va_end(ap);
        fflush(stdout);
    }
}
