/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file is the top-level implementation of the MADCAP part of MAAS.
 * The file instansiate the network socket by which client connects to.
 * The MADACP object also implements the receiving function that is
 * called whenever data is arriving from the network.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */

#ifndef _MADCAP_H_
#define _MADCAP_H_
#include <stdlib.h>
#include <stdio.h>
#include "madcap_msg_unknown.h"
#include "madcap_error.h"
#include "addr_rec.h"
#include "net_socket.h"
#include "net.h"
#include "tim.h"

typedef struct msg_id_ msg_id;

/* Struct to hold the last message sent.
 * If a dublicate message(i.e REQUEST) arrives,
 * a identical respond of the last message(i.e ACK) is
 * sent to the client.
 */
struct msg_id_ {
    uint32 xid;
    madcap_msg_type msg_type;
    madcap_msg_type response_type;
    uint8 *lease_id;
    uint16 lease_len;
    void *sent_msg;
    tim_id_t timer;
};

typedef struct madcap_ madcap;

struct madcap_ {
    net *net;
    tim *tim;
    net_socket *in_socket;
    net_socket *out_socket;
    madcap_cfg *mc_cfg;
    addr_rec *a_rec;
    madcap_plist *response_cache;
    int (*start)(madcap *mc);
    int (*save_outgoing_msg)(madcap *mc, uint32 xid,
                             madcap_msg_type msg_type,
                             madcap_msg_type resonse_type,
                             uint8 *lease_id,
                             uint16 lease_id_len,
                             void *msg);
};

/**
 * Creates a new MADCAP object.
 *
 * @param net A network abstraction object.
 * @param tim An abstract timer object.
 * @param mc_cfg The MADCAP configuration.
 * @param aap_list The list with aap's.
 * @return The newly created object. A return value of NULL
 * indicates a failure to create the new object.
 */
madcap *madcap_new(net *net, tim *tim, madcap_cfg *mc_cfg,
                   madcap_list *aap_list);

#endif
