/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains ...
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#include "madcap_msg_ack.h"
static int pub_send(madcap_msg_ack *object);
static int pub_delete(madcap_msg_ack *object);
extern madcap_error *mc_error;


/**
 * Creates a new ACK message.
 * 
 * @param n A network abstraction object.
 * @param ns An abstract network connection.
 * @param xid Transaction identifier.
 * @param data The options to the message.
 * @param data_len The length of the data in bytes.
 * @return The newly created message. The message should be sent using the send
 * function and deleted using the delete function. A return value of NULL
 * indicates a failure to create the new message.
 */
madcap_msg_ack *madcap_msg_ack_new(net *n,
                                   net_socket *ns,
                                   uint32 xid,
                                   char *data,
                                   int data_len) {
    
    madcap_msg_ack *object;

    DEBUG('m',"Creating ACK message.\n");
    
    /* Create the object and assign data to the variables in the struct. */
    object = (madcap_msg_ack *) malloc(sizeof(madcap_msg_ack));
    ASSERT(object != NULL, "Memory allocation failed.");

    object->msg         = madcap_msg_new();
    object->n           = n;
    object->ns          = ns;
    object->msg_type    = MC_ACK;
    object->addr_family = MC_ADDR_FAMILY;
    object->xid         = xid;
    object->options     = data;
    object->options_len = data_len;
    object->send        = pub_send;
    object->delete      = pub_delete;
    
    return object;    
}


/**
 * Sends an ACK message.
 *
 * @param object The message that should be sent.
 * @return 0 on success and -1 on failure.
 */
static int pub_send(madcap_msg_ack *object) {

    int result = SUCCESS;
    
    DEBUG('m',"Sending ACK message.\n");
    
    result = object->msg->create(object->msg,
                                 object->msg_type,
                                 object->addr_family,
                                 object->xid,
                                 object->options,
                                 object->options_len);
    if(result == SUCCESS) {
        result = object->msg->send(object->msg, object->n, object->ns);
    }

    if(result != SUCCESS) {
        mc_error->notify(mc_error, MC_ERR_SEND_ACK, NULL);
    }

    return result;
}


/**
 * Deletes a ACK message.
 * The function deletes a ACK message and returnes used resources to the system.
 *
 * @param object The message that should be deleted.
 * @return 0 on success and -1 on failure.
 */
static int pub_delete(madcap_msg_ack *object) {

    int result;

    result = object->msg->delete(object->msg);
    free(object);

    return result;
}
