/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains functions that is used to send an OFFER message to
 * the client as response to a DISCOVER message.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#ifndef _MADCAP_MSG_OFFER_H_
#define _MADCAP_MSG_OFFER_H_


#include <stdio.h>
#include <stdlib.h>
#include "madcap_msg.h"
#include "debug.h"
typedef struct madcap_msg_offer_ madcap_msg_offer;


/* The madcap_msg_offer object. */
struct madcap_msg_offer_ {

    /* The general MADCAP message */
    madcap_msg *msg;

    /* Variables used with madcap_msg->send(). */
    net *n;
    net_socket *ns;
    
    /* Variables used with madcap_msg->create(). */
    enum madcap_msg_type msg_type;
    uint16 addr_family;
    uint32 xid;
    char *options;
    int options_len;

    /* Function pointers. */
    int (* send) (madcap_msg_offer *object);
    int (* delete) (madcap_msg_offer *object);

};


/**
 * Creates a new OFFER message.
 * 
 * @param n A network abstraction object.
 * @param ns An abstract network connection.
 * @param xid Transaction identifier.
 * @param data The options to the message.
 * @param data_len The length of the data in bytes.
 * @return The newly created message. The message should be sent using the send
 * function and deleted using the delete function. A return value of NULL
 * indicates a failure to create the new message.
 */
madcap_msg_offer *madcap_msg_offer_new(net *n,
                                       net_socket *ns,
                                       uint32 xid,
                                       char *data,
                                       int data_len);


#endif
