/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains functions that is used when the server receives a
 * RENEW message.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#ifndef _MADCAP_MSG_RENEW_H_
#define _MADCAP_MSG_RENEW_H_


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "madcap_error.h"
#include "madcap_cfg.h"
#include "addr_rec.h"
#include "madcap_msg.h"
#include "madcap_msg_ack.h"
#include "madcap_msg_nak.h"
#include "madcap_msg_format_options.h"
#include "madcap.h"
typedef struct madcap_msg_renew_ madcap_msg_renew;


/* The madcap_msg_renew object. */
struct madcap_msg_renew_ {

    /* The general MADCAP message.*/
    madcap_msg *msg;

    /* The MADCAP object */
    madcap *mc;

    /* Variables used to process the message. */    
    uint16 opt_types_incl; /* Options included represented as binary flags. */

    /* Variables used when creating an ACK or NAK message. */
    madcap_cfg *cfg;
    addr_rec *addr;
    net *n;
    net_socket *ns;
    uint32 xid;
    uint8 *data;
    int data_len;

    /* Function pointers. */
    int (*receive)(madcap_msg_renew *object);
    int (*delete)(madcap_msg_renew *object);

};


/**
 * Creates a new RENEW message.
 * 
 * @param msg An incoming madcap_msg message with madcap_msg_type = MC_RENEW.
 * @param cfg The configuation settings for the madcap server.
 * @param addr The Madcap address record.
 * @param n A network abstraction object.
 * @param ns An abstract network connection.
 * @param mc The madcap object.
 * @return The newly created message. The message should be deleted using the
 * delete function. A return value of NULL indicates a failure to create the
 * new message.
 */
madcap_msg_renew *madcap_msg_renew_new(madcap_msg *msg,
                                       madcap_cfg *cfg,
                                       addr_rec *addr,
                                       net *n,
                                       net_socket *ns,
                                       madcap *mc);


#endif
