/*
 * This is the maasd program, a part of an implementation of the MALLOC
 * architecture.
 *
 * This file contains functions that is used when the server receives a
 * message it does not recognize.
 *
 * Copyright (C) 2001 Hkan Bystrm,Maria Isberg, Pia Lindstm,
 * Fredrik Pettersson, Linus Svensson and ke stmark.
 *
 * The project is supported by Telia Research AB, Lule, Sweden.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 1307 USA
 *
 * The authors of this program can be reached on malloc@deimos.campus.luth.se,
 * or via the project homepage at http://deimos.campus.luth.se/malloc/.
 * The Telia representative is Thomas Eriksson (Thomas.A.Eriksson@telia.se)
 */


#ifndef _MADCAP_MSG_UNKNOWN_H_
#define _MADCAP_MSG_UNKNOWN_H_


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "globals.h"
#include "madcap_error.h"
#include "madcap_msg.h"
#include "madcap_msg_ack.h"
#include "madcap_msg_nak.h"
#include "madcap_msg_format_options.h"
typedef struct madcap_msg_unknown_ madcap_msg_unknown;


/* The madcap_msg_unknown object. */
struct madcap_msg_unknown_ {

    /* The general MADCAP message.*/
    madcap_msg *msg;

    /* Variables used to process the message. */    
    uint16 opt_types_incl;         /* Options included as binary flags. */
    madcap_error_type err_type;    /* Type of error that caused this message. */
    madcap_opt_type err_type_opt;  /* Option that caused the error. */

    /* Variables used when creating an NAK message. */
    net *n;
    net_socket *ns;
    uint32 xid;
    uint8 *data;
    int data_len;

    /* Function pointers. */
    int (*receive)(madcap_msg_unknown *object);
    int (*delete)(madcap_msg_unknown *object);

};


/**
 * Creates a new UNKNOWN message.
 * 
 * @param msg An incoming madcap_msg message of unknown type.
 * @param n A network abstraction object.
 * @param ns An abstract network connection.
 * @param error The type of error that causes this message.
 * @return The newly created message. The message should be deleted using the
 * delete function. A return value of NULL indicates a failure to create the
 * new message.
 */
madcap_msg_unknown *madcap_msg_unknown_new(madcap_msg *msg,
                                           net *n,
                                           net_socket *ns,
                                           madcap_error_type error);


#endif
