#ifndef __MADCAP_PLIST_H__
#define __MADCAP_PLIST_H__

/**
 * madcap_plist.h
 *
 * Implements a priority queue.
 *
 * Methods are
 *
 * int insert(plist *p, uint32 priority, void *item);
 *
 *   Inserts an item item with priority priority into queue p.
 *
 * void *first(plist *p);
 *
 *   Returns the first item (with lowest priority) in queue p.
 *
 * int priority(plist *p);
 *
 *   Returns the priority of the frontmost item.
 *
 * int remove(plist *p);
 *
 *   Removes the frontmost item.
 *
 * void remove_element(plist *p, int (* matching_function)(void *element));
 *
 *   Removes all elements that match a given pattern. The function
 *   matching_function(), which is supplied by the user, is invoked
 *   on all elements and if the function returns != 0, the element
 *   is removed.
 *
 * int is_empty(plist *p);
 *
 *   Returns 1 if the queue is empty.
 *
 */

#include "globals.h"

typedef struct madcap_plist_ madcap_plist;

struct madcap_plist_node {
    uint32 priority;
    void *item;
    struct madcap_plist_node *next;
};


struct madcap_plist_ {
    
    struct madcap_plist_node *head;
    int len;
    
    /* these two variables are used for internal state,
       related to performance improvements */
    int last_pos;
    struct madcap_plist_node *last_node;
    
    int (* insert)(madcap_plist *p, uint32 priority, void *item);
    void * (* first)(madcap_plist *p);
    uint32 (* priority)(madcap_plist *p, int pos);
    int (* remove)(madcap_plist *p, int pos);
    
    void (* remove_element)(madcap_plist *p, int (* matching_function)
                            (void *obj, void *element), void *obj);
    
    void * (* find_element)(madcap_plist *p, int (* matching_function)
                            (void *obj, void *element), void *obj);
    
    void * (* find_elem_with_prio) (madcap_plist *p, uint32 priority);

    int (* length)(madcap_plist *p);
                            
    int (* find)(madcap_plist *p, void *item);
    void *(* elem_at)(madcap_plist *p, int pos);  
    
    int (* is_empty)(madcap_plist *p);
    
    void (* delete)(madcap_plist *p);
};

madcap_plist *madcap_plist_new();

#endif
